from loguru import logger
from tortoise import Tortoise


class Database:
    DB_URL = "sqlite://settings/database/database.db"
    DB_MODEL = {"models": ["src.utils"]}

    async def db_connect(self) -> None:
        """
        Connect to the database.
        """
        try:
            await Tortoise.init(db_url=self.DB_URL, modules=self.DB_MODEL)
            await Tortoise.generate_schemas()
        except Exception:
            logger.exception("An error occurred while connecting to the database")

    async def db_disconnect(self) -> None:
        await Tortoise.close_connections()



from loguru import logger

from sys import path
from pathlib import Path


async def load_cogs(bot) -> None:
    directory_path = Path("src")
    path.append(str(directory_path))

    for entry in directory_path.iterdir():
        for filename in entry.iterdir():
            if filename.suffix == ".py" and entry.name != "utils":
                try:
                    bot.load_extension(f"{entry.name}.{filename.stem}")
                except Exception:
                    logger.exception(f"An error occurred while loading cog {entry.name}.{filename.stem}")



from random import choices
from string import ascii_uppercase, ascii_letters, digits
from typing import List, Dict, Any, AsyncGenerator


class IssueHotline:
    MAX_STACK = 100
    """
    Hotline issue class.
    """
    async def __generate_code(self) -> str:
        part1 = "".join(choices(ascii_uppercase, k=2))
        part2 = "".join(choices(ascii_letters + digits, k=4))
        part3 = "".join(choices(ascii_letters + digits, k=4))
        return f"{part1}-{part2}-{part3}"

    async def __generate_count(
        self,
        items_list: List[Dict[str, Any]]
    ) -> AsyncGenerator[Dict[str, Any], None]:

        for item in items_list:
            item_count = item.get("m_count", 0)
            item_name = item.get("m_item", "")
            full_stacks, remainder = divmod(item_count, self.MAX_STACK)

            for _ in range(full_stacks):
                yield {"m_item": item_name, "m_count": self.MAX_STACK}

            if remainder > 0:
                yield {"m_item": item_name, "m_count": remainder}

    async def create_items(
        self,
        issue_name: str,
        items_list: List[Dict[str, Any]]
    ) -> Dict[str, Any]:
        """
        Create a dictionary with generated codes and processed items.

        items_list example: [{"m_item": "item1", "m_count": 10}, {"m_item": "item2", "m_count": 20}]
        """
        processed_items = [item async for item in self.__generate_count(items_list)]
        return {
            "m_CodeArray": [
                {
                    "m_code": await self.__generate_code(),
                    "m_name": issue_name,
                    "m_type": "item",
                    "m_itemsArray": processed_items,
                    "m_teleport_position": "",
                    "m_vehicles": {
                        "m_item": "",
                        "m_attachments": [],
                        "m_cargo": []
                    },
                    "m_give_zone_positions": [],
                    "m_give_zone_positions_forbidden": [],
                }
            ]
        }

    async def create_vehicle(
        self,
        issue_name: str,
        items_dict: Dict[str, Any]
    ) -> Dict[str, Any]:
        """
        Create a dictionary with generated code and vehicle data.

        items_dict example: {"m_item": "class_name", "m_attachments": ["class_name"], "m_cargo": null}
        """
        return {
            "m_CodeArray": [
                {
                    "m_code": await self.__generate_code(),
                    "m_name": issue_name,
                    "m_type": "vehicle",
                    "m_itemsArray": [],
                    "m_teleport_position": "",
                    "m_vehicles": items_dict,
                    "m_give_zone_positions": [],
                    "m_give_zone_positions_forbidden": [],
                }
            ]
        }

    async def create_teleport(
        self,
        issue_name: str,
        teleport_position: str
    ) -> Dict[str, Any]:
        """
        Create a dictionary with generated code and teleport data.

        teleport_position format: (x, y, z)
        """
        return {
            "m_CodeArray": [
                {
                    "m_code": await self.__generate_code(),
                    "m_name": issue_name,
                    "m_type": "teleport",
                    "m_itemsArray": [],
                    "m_teleport_position": teleport_position,
                    "m_vehicles": {
                        "m_item": "",
                        "m_attachments": []
                    },
                    "m_give_zone_position": "",
                    "m_give_zone_radius": 0,
                    "m_fresh_spawn_delay": 0
                }
            ]
        }



from typing import Optional, Dict, Any

from disnake import Embed, Forbidden, Colour
from disnake.ext import commands
from crxsnake.utils.misc import trans


class EmbedMessage:

    def __init__(self, bot: commands.Bot):
        self.bot = bot

    async def send_channel(
        self,
        channel_id: int,
        title: Optional[str] = None,
        description: Optional[str] = None,
        color: Optional[Colour] = trans,
        url: Optional[str] = None,
        image: Optional[str] = None,
        footer: Optional[str] = None,
        fields: Optional[Dict[str, Any]] = None
    ) -> None:
        """Send an Embed message to the channel"""
        channel = await self.bot.fetch_channel(channel_id)
        if channel:
            embed = self.__create_embed(
                title,
                description,
                color,
                url,
                image,
                footer,
                fields
            )
            await channel.send(embed=embed)

    async def send_user(
        self,
        user_id: int,
        title: Optional[str] = None,
        description: Optional[str] = None,
        color: Optional[Colour] = trans,
        url: Optional[str] = None,
        image: Optional[str] = None,
        footer: Optional[str] = None,
        fields: Optional[Dict[str, Any]] = None
    ) -> None:
        """Send an Embed message to the user"""
        try:
            user = await self.bot.fetch_user(user_id)
            if user:
                embed = self.__create_embed(
                    title,
                    description,
                    color,
                    url,
                    image,
                    footer,
                    fields
                )
                await user.send(embed=embed)
        except Forbidden:
            pass

    @staticmethod
    def __create_embed(
        title: Optional[str] = None,
        description: Optional[str] = None,
        color: Optional[Colour] = trans,
        url: Optional[str] = None,
        image: Optional[str] = None,
        footer: Optional[str] = None,
        fields: Optional[Dict[str, Any]] = None
    ) -> Embed:
        """Return Embed"""
        embed = Embed(
            title=title if title else "Без названия",
            description=description if description else "Описание отсутствует",
            url=url,
            color=color
        )
        if image:
            embed.set_image(url=image)
        if footer:
            embed.set_footer(text=footer)
        if fields:
            for name, value in fields.items():
                embed.add_field(name=name, value=value, inline=False)
        return embed



from .logger import logger
from .cogs import load_cogs
from .files import read_json, write_json
from .tortoise import Database
from .embed import EmbedMessage

from .utils.misc import *
from .utils.hotline import IssueHotline
from .utils.crx import IssueCRX
from .utils.convert import steam_to_be_guid, steam_to_dayz_guid


__all__ = [
    "logger",
    "load_cogs",
    "read_json",
    "write_json",
    "Database",
    "IssueHotline",
    "IssueCRX",
    "EmbedMessage",
    "steam_to_be_guid",
    "steam_to_dayz_guid"
]



from typing import List, Dict, Any


class IssueCRX:


    async def create_items(
            self, issue_type: str, items_list: List[Dict[str, Any]]
    ) -> Dict[str, List[Dict[str, Any]]]:
        """
        Create a dictionary with generated codes and processed items.
        """
        issue_dict = {"items": [], "cars": [], "sets": []}
        issue_dict[issue_type].extend(items_list)
        return issue_dict



from typing import Optional, Any
from json import loads, dumps

from aiofiles import open


async def read_json(path: str, *keys: str) -> Optional[Any]:
    """
    Read data from json file.
    """
    try:
        async with open(path, "r", encoding="utf-8") as file:
            file_content = await file.read()
            file_data = loads(file_content)

            for key in keys:
                if key not in file_data:
                    return None
                file_data = file_data[key]

            return file_data

    except Exception:
        raise


async def write_json(path: str, data: Any) -> bool:
    """
    Write data to json file.
    """
    try:
        async with open(path, "w", encoding="utf-8") as file:
            await file.write(dumps(data, ensure_ascii=False, indent=2))
            return True
    except Exception:
        raise
